@@ -10,7 +10,9 @@
 #include <strings.h>
 #include <config.h>
 #include <stdio.h>
+#include <gtk/gtkfilechooserdialog.h>
 #include <gtk/gtksignal.h>
+#include <gtk/gtkstock.h>
 #include <gtk/gtkmarshal.h>
 #include <gtk/gtktypeutils.h>
 #include <gconf/gconf-client.h>
@@ -26,7 +28,6 @@
 #include <gsdefaults.h>
 #include <ggvutils.h>
 #include <ggv-msg-window.h>
-#include <ggv-file-sel.h>
 #include <ggv-prefs.h>
 #include <ggv-prefs-ui.h>
 #include <ggv-sidebar.h>
@@ -795,7 +796,7 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
 	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
 	gint *active_rows;
-	gchar *doc, *fname, *local_path = NULL;
+	gchar *doc, *fname = NULL, *local_path = NULL;
 
 	active_rows = ggv_sidebar_get_active_list(ps_view->priv->sidebar);
 	if(active_rows[0] == -1) {
@@ -825,8 +826,25 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 	g_free(active_rows);
 
 	if(doc != NULL) {
-		fname = ggv_file_sel_request_uri(_("Select a file to save pages as"),
-										 ps_view->priv->save_path);
+		GtkWidget *chooser;
+
+		chooser = gtk_file_chooser_dialog_new(_("Select a file to save pages as"),
+						      GTK_WINDOW(gtk_widget_get_toplevel(ps_view->priv->gs)),
+						      GTK_FILE_CHOOSER_ACTION_SAVE,
+						      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+						      GTK_STOCK_SAVE, GTK_RESPONSE_OK,
+						      NULL);
+		gtk_dialog_set_default_response(GTK_DIALOG(chooser), GTK_RESPONSE_OK);
+		gtk_window_set_default_size(GTK_WINDOW(chooser), 600, 400);
+		if (ps_view->priv->save_path)
+			gtk_file_chooser_set_current_folder_uri(GTK_FILE_CHOOSER(chooser),
+						 		ps_view->priv->save_path);
+
+		if (gtk_dialog_run(GTK_DIALOG(chooser)) == GTK_RESPONSE_OK)
+			fname = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(chooser));
+
+		gtk_widget_destroy(chooser);
+
 		if(fname) {
 			if((local_path = gnome_vfs_get_local_path_from_uri(fname)) &&
 			   ggv_file_length(local_path) >= 0) {
