@@ -23,18 +23,9 @@ G_BEGIN_DECLS
 #define DEFAULT_FILE_SEL_WIDTH   320
 #define DEFAULT_FILE_SEL_HEIGHT  256
 
-typedef enum {
-        GGV_TOOLBAR_STYLE_DEFAULT = 0,
-        GGV_TOOLBAR_STYLE_BOTH = 1,
-        GGV_TOOLBAR_STYLE_ICONS = 2,
-        GGV_TOOLBAR_STYLE_TEXT = 3,
-        GGV_TOOLBAR_STYLE_LAST = 4
-} GgvToolbarStyle;
-
 extern gchar    *ggv_print_cmd;       /* print command: must print its stdin */
 extern gboolean  ggv_panel;           /* panel visible */
 extern gboolean  ggv_toolbar;         /* toolbar visible */
-extern GgvToolbarStyle  ggv_toolbar_style;  /* toolbar style */
 extern gboolean  ggv_menubar;         /* menubar visible */
 extern gboolean  ggv_statusbar;       /* statusbar visible */
 extern gboolean  ggv_save_geometry;   /* Save the current geometry for next session */
