@@ -15,9 +15,6 @@
 #include <bonobo.h>
 #include <bonobo/bonobo-ui-main.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
-#include <X11/Xlib.h>
-#include <X11/Xatom.h> 
-#include <gdk/gdkx.h>
 
 #include <math.h>
 #include <ctype.h>
@@ -26,8 +23,12 @@
 #include "gtkchecklist.h"
 #include "ggv-prefs.h"
 #include "ggv-window.h"
-#include "ggv-file-sel.h"
 #include "ggvutils.h"
+#include "ggv-recent.h"
+#include "recent-files/egg-recent-view.h"
+#include "recent-files/egg-recent-view-bonobo.h"
+#include "recent-files/egg-recent-view-gtk.h"
+#include "recent-files/egg-recent-model.h"
 
 #define GGV_POSTSCRIPT_VIEWER_CONTROL_IID "OAFIID:GNOME_GGV_Control"
 
@@ -97,51 +98,6 @@ sync_settings_popup_items(GgvWindow *win)
 	bonobo_ui_component_thaw(win->popup_uic, NULL);
 }
 
-void
-sync_toolbar_style(GgvWindow *win)
-{
-        const gchar *tbstyle;
-        gchar *gnome_tbstyle;
-
-        switch(ggv_toolbar_style) {
-        case GGV_TOOLBAR_STYLE_DEFAULT:
-                gnome_tbstyle = gconf_client_get_string(ggv_prefs_gconf_client(),
-                                                        "/desktop/gnome/interface/toolbar_style",
-                                                        NULL);
-                if(gnome_tbstyle) {
-                        if(!strcmp(gnome_tbstyle, "icons"))
-                                tbstyle = "icon";
-                        else if(!strcmp(gnome_tbstyle, "both"))
-                                tbstyle = "both";
-                        else if(!strcmp(gnome_tbstyle, "text"))
-                                tbstyle = "text";
-                        else if(!strcmp(gnome_tbstyle, "both_horiz"))
-                                tbstyle = "both_horiz";
-                        else
-                                tbstyle = "both";
-                        g_free(gnome_tbstyle);
-                }
-                else
-                        tbstyle = "both";
-                break;
-        case GGV_TOOLBAR_STYLE_BOTH:
-                tbstyle = "both";
-                break;
-        case GGV_TOOLBAR_STYLE_ICONS:
-                tbstyle = "icon";
-                break;
-        case GGV_TOOLBAR_STYLE_TEXT:
-                tbstyle = "text";
-                break;
-        default:
-                tbstyle = NULL;
-                break;
-        }
-        if(tbstyle)
-                bonobo_ui_component_set_prop(win->uic, "/Toolbar", "look",
-                                             tbstyle, NULL);
-}
-
 static void
 sync_fullscreen_items(GgvWindow *win)
 {
@@ -204,52 +160,6 @@ ggv_window_remove_control(GgvWindow *win)
         CORBA_exception_free(&ev);                
 }
 
-/* utility functions to raise window above all others! */
-#define XA_WIN_LAYER         "_WIN_LAYER"
-typedef enum {
-        WIN_LAYER_NORMAL = 4,
-        WIN_LAYER_ABOVE_DOCK = 10
-} GgvWindowLayer;
-static Atom A_XA_WIN_LAYER;
-static gboolean atom_initialized = FALSE;
-
-static void
-ggv_init_atoms()
-{
-        if(!GDK_DISPLAY() || atom_initialized) return;
-        A_XA_WIN_LAYER = XInternAtom(GDK_DISPLAY(), XA_WIN_LAYER, False);
-        atom_initialized = TRUE;
-}
-
-static void
-ggv_window_set_layer(GgvWindow *win, GgvWindowLayer layer)
-{
-        if(!atom_initialized)
-                ggv_init_atoms();
-        if(GTK_WIDGET_MAPPED(win)) {
-                XEvent e;
-                e.type = ClientMessage;
-                e.xclient.type = ClientMessage;
-                e.xclient.window = GDK_WINDOW_XID(GTK_WIDGET(win)->window);
-                e.xclient.message_type = A_XA_WIN_LAYER;
-                e.xclient.format = 32;
-                e.xclient.data.l[0] = (long)layer;
-                e.xclient.data.l[1] = GDK_CURRENT_TIME;
-
-                XSendEvent(GDK_DISPLAY(), GDK_ROOT_WINDOW(), False,
-                           SubstructureNotifyMask, (XEvent *)&e);
-        }
-        else {
-                long data[1];
-
-                data[0] = layer;
-                XChangeProperty(GDK_DISPLAY(),
-                                GDK_WINDOW_XID(GTK_WIDGET(win)->window),
-                                A_XA_WIN_LAYER, XA_CARDINAL, 32,
-                                PropModeReplace, (unsigned char *)data, 1);
-        }
-}
-
 static void
 ggv_window_set_fullscreen(GgvWindow *win, gboolean fs)
 {
@@ -290,6 +200,7 @@ ggv_window_set_fullscreen(GgvWindow *win, gboolean fs)
                 gtk_window_resize(GTK_WINDOW(win),
                                   gdk_screen_width(),
                                   gdk_screen_height());
+		gtk_window_fullscreen (GTK_WINDOW(win));
         }
         else {
                 win->show_menus = win->orig_sm;
@@ -313,8 +224,8 @@ ggv_window_set_fullscreen(GgvWindow *win, gboolean fs)
                 }
                 gtk_window_move(GTK_WINDOW(win), win->orig_x, win->orig_y);
                 gtk_window_resize(GTK_WINDOW(win), win->orig_width, win->orig_height);
+		gtk_window_unfullscreen (GTK_WINDOW(win));
         }
-        ggv_window_set_layer(win, fs?WIN_LAYER_ABOVE_DOCK:WIN_LAYER_NORMAL);
         sync_settings_menu_items(win);
         sync_settings_popup_items(win);
         sync_fullscreen_items(win);
@@ -428,11 +339,30 @@ control_property_changed_handler(BonoboListener    *listener,
 static void
 verb_FileOpen(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
+        EggRecentItem *item;
+        EggRecentModel *recent;
+
         GgvWindow *win = GGV_WINDOW(data);
-        gchar *fname, *escaped_fname;
+        gchar *fname = NULL, *escaped_fname;
+	GtkWidget *chooser;
+
+	chooser = gtk_file_chooser_dialog_new(_("Select a PostScript document"),
+					      GTK_WINDOW(win),
+					      GTK_FILE_CHOOSER_ACTION_OPEN,
+					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					      GTK_STOCK_OPEN, GTK_RESPONSE_OK,
+					      NULL);
+	gtk_dialog_set_default_response(GTK_DIALOG(chooser), GTK_RESPONSE_OK);
+	gtk_window_set_default_size(GTK_WINDOW(chooser), 600, 400);
+	if (win->filename)
+		gtk_file_chooser_set_uri(GTK_FILE_CHOOSER(chooser),
+					 win->filename);
+
+	if (gtk_dialog_run(GTK_DIALOG(chooser)) == GTK_RESPONSE_OK)
+		fname = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(chooser));
+
+	gtk_widget_destroy(chooser);
 
-        fname = ggv_file_sel_request_uri(_("Select a PostScript document"),
-                                         win->filename);
         if(fname) {
                 if(!ggv_window_load(win, fname)) {
                         GtkWidget *dlg;
@@ -446,6 +376,12 @@ verb_FileOpen(BonoboUIComponent *uic, gpointer data, const char *cname)
                         gtk_dialog_run(GTK_DIALOG(dlg));
                         gtk_widget_destroy(dlg);
                 }
+		recent = ggv_recent_get_model ();
+		item = egg_recent_item_new_from_uri (fname);
+		egg_recent_item_add_group (item, "ggv");
+		egg_recent_model_add_full (recent, item);
+		egg_recent_item_unref (item);
+
                 g_free(fname);
         }
 }
@@ -645,7 +581,7 @@ verb_HelpAbout(BonoboUIComponent *uic, gpointer data, const char *cname)
 		about = gnome_about_new (
 			_("Gnome Ghostview"),
 			VERSION,
-			"Copyright \xc2\xa9 1998-2002 Free Software Foundation, Inc.",
+			"Copyright \xc2\xa9 1998-2004 Free Software Foundation, Inc.",
 			_("The GNOME PostScript document previewer"),
 			authors,
 			documenters,
@@ -815,15 +751,7 @@ static void
 prefs_changed(GConfClient* client, guint cnxn_id,
               GConfEntry *entry, gpointer user_data)
 {
-        if(!strcmp(entry->key, "/desktop/gnome/interface/toolbar_style") &&
-           ggv_toolbar_style == GGV_TOOLBAR_STYLE_DEFAULT) {
-                sync_toolbar_style(GGV_WINDOW(user_data));
-        }
-        else if(!strcmp(entry->key, "/apps/ggv/layout/toolbarstyle")) {
-                ggv_toolbar_style = gconf_value_get_int(entry->value);
-                sync_toolbar_style(GGV_WINDOW(user_data));
-        }
-        else if(!strcmp(entry->key, "/apps/ggv/layout/showtoolbar")) {
+        if(!strcmp(entry->key, "/apps/ggv/layout/showtoolbar")) {
                 GgvWindow *window = GGV_WINDOW(user_data);
                 gboolean state_f = gconf_value_get_bool(entry->value);
                 if(window->show_toolbar != state_f) {
@@ -924,6 +852,124 @@ ggv_window_get_type (void)
 	return ggv_window_type;
 }
 
+static void
+menu_position_under_widget (GtkMenu *menu, int *x, int *y,
+                            gboolean *push_in, gpointer user_data)
+{
+        GtkWidget *w;
+        int screen_width, screen_height;
+        GtkRequisition requisition;
+                                                                                                                             
+        w = GTK_WIDGET (user_data);
+                                                                                                                             
+        gdk_window_get_origin (w->window, x, y);
+        *x += w->allocation.x;
+        *y += w->allocation.y + w->allocation.height;
+                                                                                                                             
+        gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
+                                                                                                                             
+        screen_width = gdk_screen_width ();
+        screen_height = gdk_screen_height ();
+                                                                                                                             
+        *x = CLAMP (*x, 0, MAX (0, screen_width - requisition.width));
+        *y = CLAMP (*y, 0, MAX (0, screen_height - requisition.height));
+}
+
+static gboolean
+open_button_pressed_cb (GtkWidget *widget,
+                        GdkEventButton *event,
+                        gpointer *user_data)
+{
+        GtkWidget *menu;
+                                                                                                                             
+        g_return_val_if_fail (GTK_IS_BUTTON (widget), FALSE); 
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE); 
+
+	menu = g_object_get_data (G_OBJECT (widget), "recent-menu");
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, menu_position_under_widget, widget,
+			event->button, event->time);
+ 
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
+        return TRUE;
+}
+
+static gboolean
+open_button_key_pressed_cb (GtkWidget *widget,
+                            GdkEventKey *event,
+                            gpointer *user_data)
+{
+        if (event->keyval == GDK_space ||
+            event->keyval == GDK_KP_Space ||
+            event->keyval == GDK_Return ||
+            event->keyval == GDK_KP_Enter) {
+                open_button_pressed_cb (widget, NULL, user_data);
+        }
+                                                                                                                             
+        return FALSE;
+}
+
+static gboolean
+ggv_file_open_recent (EggRecentView *view, EggRecentItem *item, gpointer data)
+{
+        GtkWidget *newwin;
+        GgvWindow *win = GGV_WINDOW (data);
+        gchar *uri_utf8;
+                                                                                                                             
+        uri_utf8 = egg_recent_item_get_uri_utf8 (item);
+                                                                                                                             
+        if(win->current_page < 0)
+                newwin = GTK_WIDGET (win);
+        else
+                newwin = NULL;
+                                                                                                                             
+        if (newwin == NULL) {
+                newwin = ggv_window_new ();
+                gtk_widget_show (GTK_WIDGET (newwin));
+        }
+                                                                                                                             
+        ggv_window_load (GGV_WINDOW (newwin), uri_utf8);
+        newwin = NULL;
+                                                                                                                             
+        return TRUE;
+}
+
+ggv_add_open_button (GgvWindow *win, BonoboUIComponent *ui_component,
+                     const gchar *path, const gchar *tooltip)
+{
+        GtkWidget *menu;
+        EggRecentViewGtk *view;
+        EggRecentModel *model;
+        GtkWidget *button;
+                                                                                                                             
+        button = gtk_toggle_button_new ();
+        gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+                                                                                                                             
+        gtk_container_add (GTK_CONTAINER (button),
+                           gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_OUT));
+                                                                                                                             
+        gtk_widget_show_all (GTK_WIDGET (button));
+                                                                                                                             
+        model = ggv_recent_get_model ();
+                                                                                                                             
+        menu = gtk_menu_new ();
+        gtk_widget_show (menu);
+        view = egg_recent_view_gtk_new (menu, NULL);
+        g_signal_connect (view, "activate",
+                          G_CALLBACK (ggv_file_open_recent), win);
+        egg_recent_view_gtk_show_icons (view, TRUE);
+        egg_recent_view_gtk_show_numbers (view, FALSE);
+        egg_recent_view_set_model (EGG_RECENT_VIEW (view), model);
+        g_object_set_data (G_OBJECT (button), "recent-menu", menu);
+        g_signal_connect_object (button, "key_press_event",
+                                 G_CALLBACK (open_button_key_pressed_cb),
+                                 win, 0);
+        g_signal_connect_object (button, "button_press_event",
+                                 G_CALLBACK (open_button_pressed_cb),
+                                 win, 0);
+        bonobo_ui_component_widget_set (ui_component, path, GTK_WIDGET (button),
+                                        NULL);
+}
+
 GtkWidget *
 ggv_window_new(void)
 {
@@ -933,6 +979,8 @@ ggv_window_new(void)
         CORBA_Environment ev;
         Bonobo_Control control;
         GtkWidget *widget;
+        EggRecentViewBonobo *view;
+        EggRecentModel *model;
         Bonobo_PropertyBag pb;
         gchar *mask;
 
@@ -994,8 +1042,12 @@ ggv_window_new(void)
         win->uic = uic;
 	bonobo_ui_component_set_container(uic, BONOBO_OBJREF(ui_container),
                                           NULL);
+
+
 	bonobo_ui_util_set_ui(uic, DATADIR, "ggv-ui.xml", "GGV", NULL);
-        sync_toolbar_style(win);
+
+        ggv_add_open_button (win, uic, "/Toolbar/TBOpenMenu", _("Open a file."));
+
 	bonobo_ui_component_add_verb_list_with_data(uic, ggv_app_verbs, win);
 	bonobo_ui_component_add_listener(uic, "SettingsShowMenus",
                                          listener_SettingsShow, win);
@@ -1007,8 +1059,17 @@ ggv_window_new(void)
                                          listener_SettingsShow, win);
 	bonobo_ui_component_add_listener(uic, "ViewFullscreen",
                                          listener_ViewFullscreen, win);
+
         sync_settings_menu_items(win);
 
+        model = ggv_recent_get_model ();
+        view = egg_recent_view_bonobo_new (uic, "/menu/File/Recents");
+        egg_recent_view_bonobo_show_icons (EGG_RECENT_VIEW_BONOBO (view), FALSE);
+        egg_recent_view_set_model (EGG_RECENT_VIEW (view), model);
+                                                                                                                             
+        g_signal_connect (G_OBJECT (view), "activate",
+                          G_CALLBACK (ggv_file_open_recent), win);
+                                                                                                                             
         if(!win->show_toolbar)
                 bonobo_ui_component_set_prop(uic, "/Toolbar", "hidden", "1", NULL);
         if(!win->show_menus)
@@ -1082,10 +1143,6 @@ ggv_window_new(void)
                              ggv_default_width, ggv_default_height);
         gtk_window_set_policy(GTK_WINDOW(win), TRUE, TRUE, FALSE);
 
-        gconf_client_notify_add(ggv_prefs_gconf_client(),
-                                "/desktop/gnome/interface/toolbar_style",
-                                (GConfClientNotifyFunc)prefs_changed,
-                                win, NULL, NULL);
         gconf_client_notify_add(ggv_prefs_gconf_client(),
                                 "/apps/ggv/layout",
                                 (GConfClientNotifyFunc)prefs_changed,
@@ -1171,21 +1228,6 @@ ggv_window_close(GgvWindow *win)
                 bonobo_main_quit();
 }
 
-void
-ggv_window_apply_prefs(GgvWindow *win)
-{
-        if(win) {
-                sync_toolbar_style(win);
-        }
-        else {
-                GList *node = window_list;
-                while(node) {
-                        ggv_window_apply_prefs(GGV_WINDOW(node->data));
-                        node = node->next;
-                }
-        }
-}
-
 const GList *
 ggv_get_window_list()
 {
