@@ -152,7 +152,6 @@ static gboolean broken_pipe = FALSE;
 
 static void catchPipe(int i)
 {
-        g_message("in catch Pipe!!!\n");
         broken_pipe = True;
 }
 
@@ -672,6 +671,7 @@ gtk_gs_size_allocate (GtkWidget     *widget,
                 gtk_gs_set_zoom(gs, 0.0);
         }
 #endif
+
         if (GTK_WIDGET_REALIZED(gs)) {
                 gtk_gs_munge_adjustments (gs);
         }
@@ -1278,7 +1278,8 @@ check_filecompressed (GtkGS *gs)
 			cmd = gtk_gs_defaults_get_unbzip2_cmd();
 		}
 	}
-	fclose (file);
+        if(NULL != file)
+                fclose (file);
 
 	if (!cmd)
 		return gs->gs_filename;
@@ -1373,8 +1374,6 @@ check_pdf (GtkGS *gs)
 		cmdline = g_strdup_printf ("%s >%s 2>&1", cmd, filename_err);
 		g_free (cmd);
 
-                g_message("cmdline: %s", cmdline);
-
 		if ((system (cmdline) == 0) &&
                     ggv_file_readable (filename_dsc) ) {
 
@@ -1722,8 +1721,6 @@ gtk_gs_load (GtkGS *gs, const gchar *fname)
 		return FALSE;
 	}
 
-        g_message("File: %s\n", fname);
-
  	/* prepare this document */
 
  	/* default values: no dsc information available  */
@@ -2172,11 +2169,12 @@ gtk_gs_zoom_to_fit(GtkGS *gs, gboolean fit_width)
 
         avail_w = (gs->avail_w > 0)?gs->avail_w:gs->width;
         avail_h = (gs->avail_h > 0)?gs->avail_h:gs->height;
-	new_zoom = (gfloat)avail_w/(gfloat)gs->width*gs->zoom_factor;
+
+        new_zoom = ((gfloat)avail_w)/((gfloat)gs->width)*gs->zoom_factor;
 	if(!fit_width) {
-		new_y = new_zoom*(gfloat)gs->width/gs->zoom_factor;
+		new_y = new_zoom*((gfloat)gs->height)/gs->zoom_factor;
 		if(new_y > avail_h)
-			new_zoom = (gfloat)avail_h/(gfloat)gs->height*gs->zoom_factor;
+			new_zoom = ((gfloat)avail_h)/((gfloat)gs->height)*gs->zoom_factor;
 	}
 
         return new_zoom;
@@ -2627,7 +2625,6 @@ gtk_gs_get_postscript(GtkGS *gs, gint *pages)
                                            gs->gs_filename);
                 cmd = g_strdup_printf(gtk_gs_defaults_get_convert_pdf_cmd(),
                                       tmpn, fname);
-                g_message("Cmdline 2 %s", cmd);
                 g_free(fname);
                 if((system(cmd) == 0) && ggv_file_readable(tmpn)) {
                         GtkWidget *tmp_gs;
